<?php
/*******************************************************************************
*	This file is part of Woogle4MediaWiki
*   (http://www.mediawiki.org/wiki/Extension:Woogle4MediaWiki)
*
*	Copyright (c) 2007 - 2009 Hans-Jörg Happel and
*	FZI Forschungszentrum Informatik and der Universität Karlsruhe (TH)
*
*   Woogle4MediaWiki is free software: you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   Woogle4MediaWiki is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with Woogle4MediaWiki.  If not, see <http://www.gnu.org/licenses/>.
*******************************************************************************/ 

/**
 *	This file contains Woogle-specific utility functions
 */
class WoogleUtil {
	
	// singletons
	private static $queryService;
	private static $queryManager;
	private static $indexService;
		
	// TODO temporarily disabled (http://octopus13.fzi.de:8080/browse/WOO-148)
	//private static $specialPages = array('Search', 'Statistics', 'Feedback', 'Config', 'ResultInfo');
	private static $specialPages = array('Search', 'Config', 'Feedback', 'Statistics', 'Help');
	
	private static $messagesLoaded = false;
	
			
	//
	// Utility methods for profiling and logging
	//
	
	
	/**
	 * Wrapper method for profiling - to be called at the start of a method/function
	 * @param		$name	name of the profiled method
	 * @return		void
	 */
	public static function profileIn($name) {
		if (WoogleConfig::useProfiling()) wfProfileIn('Woogle ' . $name);
	}
	
	/**
	 * Wrapper method for profiling - to be called at the end of a method/function
	 * @param		$name		name of the profiled method
	 * @param		$return		return value (optional)
	 * @return		void
	 */
	public static function profileOut($name, $return = true) {
		if (WoogleConfig::useProfiling()) wfProfileOut('Woogle ' . $name);
		return $return;
	}
	
	/**
	 * @see profileIn
	 */
	public static function addonProfileIn($addon, $name) {
		if (WoogleConfig::useProfiling()) wfProfileIn('Woogle' . '::' . $addon . ' ' . $name);
	}

	/**
	 * @see profileOut
	 */
	public static function addonProfileOut($addon, $name, $return = true) {
		if (WoogleConfig::useProfiling()) wfProfileOut('Woogle' . '::' . $addon . ' ' . $name);
		return $return;
	}
	
	
		
	private static function writeLog($msg, $file){
		WoogleUtil::profileIn(__METHOD__);
		
		if (WoogleConfig::get('useLogging', false)){
			$prefix 	= '[' . date('Y-m-d H:i:s') . '] ';
			$logline 	= $prefix . $msg . "\n";
			
			//write logs to shell (typically in testing mode)
			if (WoogleConfig::get('writeLogsToShell', false)){
				echo $logline;				
			} else {
				// write logs to file
				$dFile = fopen(WoogleUtil::getLogPath() . $file, 'a');
				fwrite($dFile, $logline);
				fclose($dFile);
			}
		}
		WoogleUtil::profileOut(__METHOD__);
	}
	
	public static function debug($msg) {
		if (WoogleConfig::get('useDebugLogging', false)) self::writeLog($msg, 'debug.log');
	}
	
	public static function info($msg){
		self::writeLog($msg, 'info.log');
	}

	public static function error($msg){
		self::writeLog($msg, 'error.log');
	}
	
	public static function feedback($msg){
		self::writeLog($msg, 'feedback.log');
	}
	
	public static function log_query($msg){
		self::writeLog(';' . self::getAnonUserID() . ';' . $msg, 'query.log');
	}
	
	public static function perflog($msg, $logtype){
		if (WoogleConfig::get('useLogging') && WoogleConfig::get('usePerformanceLogging')){
			$lfile = self::getLogPath() . 'performance_' . $logtype . '.log';
			
			// write instrumentation log entry
			if ($fh = fopen($lfile,'a')) {
				$text =  date('Y-m-d;H:i:s') . ';' . self::getQueryService()->isNative() . ';' . $msg;
				fwrite($fh,"$text\n");
				fclose($fh);
			} else {
				self::error("Could not create peformance log");
			}
		}
	}
	
	/**
	 * @return 	string, last 10 lines of error log
	 */
	public static function getErrors(){
		WoogleUtil::profileIn(__METHOD__);
		$efile = self::getLogPath() . 'error.log';
		$res = '';

		$i=0;
		$content = array();
		if (file_exists($efile)){
			$dFile = fopen($efile, 'r');
			while (($line = fgets($dFile)) != null ){
				$content[$i] = $line;
				$i++;
			}
			fclose($dFile);
			
			// read last 50 lines
			$numlines=0;
			for($count = count($content)-1; $count >= 0; $count--){
				$res .= $content[$count] . '<br />';
				$numlines++;
				if ($numlines > 49) break; 
    		}
			
		}
		return WoogleUtil::profileOut(__METHOD__, $res);
	}
	
	//
	// Access to Woogle runtime singleton object references
	//
	
	/**
	 * Returns singleton instance of WoogleQueryService
	 * @return Object, instance of WoogleQueryService
	 */
	public static function getQueryService() {
		WoogleUtil::profileIn(__METHOD__);
		if (self::$queryService == null) {
			$class = 'WoogleQueryService' . WoogleConfig::get('indexAddon');
			self::$queryService = new $class();
		}
		WoogleUtil::profileOut(__METHOD__);
		return self::$queryService;
	}
	
	/**
	 * Returns singleton instance of WoogleQueryManager
	 * @return Object, instance of WoogleQueryManager
	 */
	public static function getQueryManager() {
		WoogleUtil::profileIn(__METHOD__);
		if (self::$queryManager == null) {
			$class = 'WoogleQueryManager' . WoogleConfig::get('indexAddon');
			self::$queryManager = new $class();
		}
		WoogleUtil::profileOut(__METHOD__);
		return self::$queryManager;
	}
	
	/**
	 * Returns singleton instance of WoogleIndexService
	 * @return Object, instance of WoogleIndexService
	 */
	public static function getIndexService() {
		WoogleUtil::profileIn(__METHOD__);
		if (self::$indexService == null) {
			$class = 'WoogleIndexService' . WoogleConfig::get('indexAddon');
			self::$indexService = new $class();
		}
		WoogleUtil::profileOut(__METHOD__);
		return self::$indexService;
	}
	
	

	//
	// E-Mail
	//

	/**
	 * Send a feedback mail to the Woogle admin
	 * 
	 * @param $subject			String, E-Mail subject
	 * @param $body				String, E-Mail body
	 * @return					boolean, success
	 */
	public static function sendFeedbackMail($subject, $body){
		global $wgEmergencyContact, $wgSitename;
		WoogleUtil::profileIn(__METHOD__);
				
		if (WoogleConfig::get('useEMailLogging')){
			$to = WoogleConfig::get('adminEMail');
			if ($to == '') $to = $wgEmergencyContact;
	
			WoogleUtil::debug('im If-Block');
			$subject = '['.$wgSitename.'] ' . $subject;
			
			$res = WoogleMWUtil::sendMail($to, $subject, $body);
		} else {
			$res = true;
		}
			
		return WoogleUtil::profileOut(__METHOD__, $res);
	
	}
	
	/**
	 * Send an error mail to the Woogle admin
	 * 
	 * @param $subject			String, E-Mail subject
	 * @param $body				String, E-Mail body
	 * @return					boolean, success
	 */
	public static function sendErrorMail($subject, $body){
		global $wgEmergencyContact;
		WoogleUtil::profileIn(__METHOD__);
		
		if (WoogleConfig::get('useEMailLogging')){
			$to = WoogleConfig::get('adminEMail');
			if ($to == '') $to = $wgEmergencyContact;
	
			$subject = '[Woogle Error] ' . $subject;
			
			$res = WoogleMWUtil::sendMail($to, $subject, $body);
		} else {
			$res = true;
		}
			
		return WoogleUtil::profileOut(__METHOD__, $res);
	
	}

	
	//
	//	Utility methods for Woogle special pages
	//

	
	public static function getSpecialPageClass($specialPage) {
		WoogleUtil::profileIn(__METHOD__);
		return WoogleUtil::profileOut(__METHOD__, 'WoogleSpecial' . $specialPage);
	}
	
	public static function getSpecialPageTitle($specialPage) {
		WoogleUtil::profileIn(__METHOD__);
		return WoogleUtil::profileOut(__METHOD__, SpecialPage::getTitleFor(self::getSpecialPageClass($specialPage)));
	}
	
	/**
	 * Looks up translation from WoogleLang::getSpecialPages()
	 * 
	 * @param 		String, key for special page
	 * @return 		String, localized value
	 */
	public static function getSpecialPageLocal($specialPage) {
		WoogleUtil::profileIn(__METHOD__);
		return WoogleUtil::profileOut(__METHOD__, wfMsgForContent('woogleSpecial' . $specialPage));
	}
	
	public static function addSpecialPage($name) {
		// no profiling, as wfProfileIn/Out aren't known yet
		self::$specialPages[] = $name;
	}
	
	public static function getSpecialPages() {
		// no profiling, as wfProfileIn/Out aren't known yet
		return self::$specialPages;
	}
	
		
	//
	// Localization
	//


	/**
	 * Initialize localized strings / loading into MediaWiki message cache 
	 * @return 	true
	 */
	public static function loadMessages() {
		global $wgMessageCache;
		WoogleUtil::profileIn(__METHOD__);
		if (self::$messagesLoaded) {
			return true;
		}
		self::$messagesLoaded = true;

		$woogleMessages = array();
		if (file_exists(dirname(__FILE__) . '/../languages') && is_dir(dirname(__FILE__) . '/../languages')) {
			$woogleLanguages = opendir(dirname(__FILE__) . '/../languages');
			while (false !== ($file = readdir($woogleLanguages))) {
				if (is_file(dirname(__FILE__) . '/../languages/' . $file)) {
					require_once(dirname(__FILE__) . '/../languages/' . $file);
					$class = substr($file, 0, strlen($file) - 4);
					if (class_exists($class)) {
						$lang = new $class();
						foreach ($lang->getMessages() as $msgKey => $msgValue) {
							$woogleMessages[$lang->getIsoCode()]['woogle' . $msgKey] = $msgValue;
						}
						foreach ($lang->getSpecialPages() as $specialKey => $specialValue) {
							$woogleMessages[$lang->getIsoCode()]['woogleSpecial' . $specialKey] = $specialValue[0];
							$woogleMessages[$lang->getIsoCode()][strtolower($specialValue[0])] = $specialValue[1];
						}
					}
				}
			}
		}
		#wfRunHooks('WoogleMessages', array(&$woogleMessages));
		foreach ($woogleMessages as $lang => $langMessages) {
			$wgMessageCache->addMessages($langMessages, $lang);
		}
		WoogleUtil::profileOut(__METHOD__);
		return true;
	}
	
	/**
	 * Get namespaces and labels in appropriate language
	 * @return	array of namespace/label strings
	 */
	public static function getNamespaces(){
		global $wgLanguageCode;
		WoogleUtil::profileIn(__METHOD__);
		
		$res 	= array();
		$res_en = array();
		
		if (file_exists(dirname(__FILE__) . '/../languages') && is_dir(dirname(__FILE__) . '/../languages')) {
			$woogleLanguages = opendir(dirname(__FILE__) . '/../languages');
			while (false !== ($file = readdir($woogleLanguages))) {
				if (is_file(dirname(__FILE__) . '/../languages/' . $file)) {
					require_once(dirname(__FILE__) . '/../languages/' . $file);
					$class = substr($file, 0, strlen($file) - 4);
					if (class_exists($class)) {
						$lang = new $class();
						if ( strcasecmp($lang->getIsoCode(), $wgLanguageCode) == 0 )  $res_en = $lang->getNamespaces();
						if ( strcasecmp($lang->getIsoCode(), 'en') == 0 ) $res_en = $lang->getNamespaces();
					}
				}
			}
		}
			
		// take english as default
		if (count($res) == 0) $res = $res_en;
		
		return $res;
		WoogleUtil::profileOut(__METHOD__);
	}
	
	/**
	 * Get localized string for a given key
	 * 
	 * @param 		$key	message key
	 * @return 		string, localized message
	 */
	public static function getMsg($key) {
		WoogleUtil::profileIn(__METHOD__);
		$args = func_get_args();
		array_shift($args);
		WoogleUtil::loadMessages();
		return WoogleUtil::profileOut(__METHOD__, wfMsgReal('woogle' . $key, $args));
	}

	
		
	//
	// Path methods 
	//
	
	
	/**
	 * @return 	String, path of directory where log files are written
	 * 
	 * Example: "R:\dev\htdocs\wiki\extensions\Woogle\logs\"
	 */
	public static function getLogPath() {
		WoogleUtil::profileIn(__METHOD__);
		return WoogleUtil::profileOut(__METHOD__, dirname(__FILE__) . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'logs' . DIRECTORY_SEPARATOR);
	}
	
	/**
	 * @return 	String, path of directory where the template-files are located
	 * 
	 * Example: "R:\dev\htdocs\wiki\extensions\Woogle\resources\tpl\"
	 */
	public static function getTplFolder() {
		WoogleUtil::profileIn(__METHOD__);
		return WoogleUtil::profileOut(__METHOD__, dirname(__FILE__) . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'resources' . DIRECTORY_SEPARATOR.'tpl'.DIRECTORY_SEPARATOR);		
	}
	
	/*
	 * @return 		String, Example: "/wiki/extensions/Woogle/"
	 */
	public static function getWooglePath() {
		WoogleUtil::profileIn(__METHOD__);
		
		$epath = dirname( __FILE__ );
		$epath = substr( $epath , strpos($epath, 'extensions'), strpos($epath, 'WoogleUtil.php')-8);
		$epath = str_replace('\\', '/', $epath);
		return WoogleUtil::profileOut(__METHOD__, WoogleMWUtil::getWikiAbsolutePath() . $epath);
	}
	
	
	/*
	 * @return 		String, Example: "http://localhost/wiki/extensions/Woogle/"
	 */
	public static function getWoogleAbsolutePath() {
		WoogleUtil::profileIn(__METHOD__);

		$epath = dirname( __FILE__ );
		$epath = substr( $epath , strpos($epath, 'extensions'), strpos($epath, 'WoogleUtil.php')-8);
		$epath = str_replace('\\', '/', $epath);
		return WoogleUtil::profileOut(__METHOD__, WoogleMWUtil::getWikiAbsolutePath() . $epath);
	}
	
	public static function getImagePath() {
		WoogleUtil::profileIn(__METHOD__);
		return WoogleUtil::profileOut(__METHOD__, WoogleUtil::getWoogleAbsolutePath() . 'resources/img/');
	}
	
		/**
	 * 
	 * @return array(
            "browser"      => Browser,
	        "version"      => Version,
	        "platform"     => OS,
	        "userAgent"    => UserAgent);
        );
	 */
	public static function getUserBrowser() {
		
		$browsers = "mozilla msie gecko firefox ";
		$browsers.= "konqueror safari netscape navigator ";
		$browsers.= "opera mosaic lynx amaya omniweb";
		
		$browsers = split(" ", $browsers);
		
		$userAgent = strToLower( $_SERVER['HTTP_USER_AGENT']);
		
		$l = strlen($userAgent);
		for ($i=0; $i<count($browsers); $i++){
		  $browser = $browsers[$i];
		  $n = stristr($userAgent, $browser);
		  if(strlen($n)>0){
		    $version = "";
		    $navigator = $browser;
		    $j=strpos($userAgent, $navigator)+$n+strlen($navigator)+1;
		    for (; $j<=$l; $j++){
		      $s = substr ($userAgent, $j, 1);
		      if(is_numeric($version.$s) )
		      $version .= $s;
		      else
		      break;
		    }
		  }
		}
		
	    if (strpos($userAgent, 'linux')) {
	        $platform = 'linux';
	    }
	    else if (strpos($userAgent, 'macintosh') || strpos($userAgent, 'mac platform x')) {
	        $platform = 'mac';
	    }
	    else if (strpos($userAgent, 'windows') || strpos($userAgent, 'win32')) {
	        $platform = 'windows';
	    }
	
		return array(
	        "browser"      => $navigator,
	        "version"      => $version,
	        "platform"     => $platform,
	        "userAgent"    => $userAgent);
		
	}
	
	//
	//	Get links
	//
	
	
	/**
	 * 
	 * @return 		String, valid LinkPrefix - either advanced or non-advanced
	 */
	public static function getWoogleLinkPrefix() {
		WoogleUtil::profileIn(__METHOD__);
		
		if (WoogleConfig::get('advanced')) {
			return WoogleUtil::profileOut(__METHOD__, self::getWoogleLinkPrefixAdvanced());
		} else {
			return WoogleUtil::profileOut(__METHOD__, self::getWoogleLinkPrefixNonadvanced());
		}
	}
	
	/*
	 * @return 		String, Example: "/wiki/index.php/Woogle:"
	 */
	public static function getWoogleLinkPrefixAdvanced() {
		WoogleUtil::profileIn(__METHOD__);
		WoogleUtil::loadMessages();
		return WoogleUtil::profileOut(__METHOD__, WoogleMWUtil::getArticlePath() . WoogleUtil::getSpecialPageLocal('Search') . ':');
	}

	/*
	 * @return 		String, Example: "/wiki/index.php/Special:Woogle/" 
	 */
	public static function getWoogleLinkPrefixNonadvanced() {
		WoogleUtil::profileIn(__METHOD__);
		return WoogleUtil::profileOut(__METHOD__, WoogleMWUtil::getArticlePath() . self::getWoogleSearchSpecialPageName() . '/');
	}
	
	/*
	 * @return 		String, Example: "Special:Woogle"
	 */
	public static function getWoogleSearchSpecialPageName() {
		WoogleUtil::profileIn(__METHOD__);
		WoogleUtil::loadMessages();
		return WoogleUtil::profileOut(__METHOD__, WoogleUtil::getSpecialPageTitle('Search')->getPrefixedText());
	}
	
	/**
	 * Removes field f from a string
	 * 
	 * @param $fields	string, request string with fields and values assigned
	 * @param $f		string, field name
	 * @return 			string, *without* field name and value for f
	 */
	public static function extractField($fields, $f) {
		WoogleUtil::profileIn(__METHOD__);
		$newFields = "";
		$requestFields = explode("&", $fields);
		foreach ($requestFields as $field) {
			$pair = explode("=", $field);

			// add to newfields if not equal to f
			if ($pair[0] != $f) {
				// add ampersand for subsequent fields (but not for the first)
				if (!empty($newFields)) {
					$newFields .= '&';
				}
				$newFields .= $field;
			}
		}
		return WoogleUtil::profileOut(__METHOD__, $newFields);
	}
	
	/**
	 * Converts MediaWiki articles to WoogleDocuments
	 * 
	 * @param $article			Instance of a MediaWiki article object
	 * @param $content			String, optional alternative content fulltext for article
	 * @return					WoogleDocument
	 */
	public static function articleToDocument($article, $content = -1){
		$wd = new WoogleDocument();
		
		// fill content
		if ($content == -1) {
			$content = $article->getContent();
			if ( WoogleConfig::get('htmlIndexing') ){
				$content = WoogleMWUtil::parseArticle($article, true);
				$content = strip_tags($content);
			} else {
				$content = WoogleMWUtil::stripMediaWikiMarkup($content);
			}
		}
		
		$wd->setId(WoogleQueryServiceCommon::getId($article->getID()));
		$wd->setShortId($article->getID());
		$wd->setUri($article->getTitle()->getFullURL()); // vs. WoogleMWUtil::getArticleAbsolutePath() . $tt->getPartialURL()
		$wd->setLabel($article->getTitle()->getFullText());
		$wd->setContent($content);
		$wd->setChanged($article->getTimestamp());
		$wd->setCreated($article->getTimestamp());	// TODO - wrong!
		
		return $wd;
	}
	
	
	public static function longToString($long) {
		WoogleUtil::profileIn(__METHOD__);
		return WoogleUtil::profileOut(__METHOD__, sprintf('%010d', $long));
	}
	
	public static function stringToLong($string) {
		WoogleUtil::profileIn(__METHOD__);
		return WoogleUtil::profileOut(__METHOD__, intval($string));
	}
	
	public static function getSecretKey(){
		global $wgSecretKey, $wgDBpassword, $wgDBname;
		$res = $wgSecretKey;
		if (strlen($res < 1)) $res = $wgDBpassword;
		if (strlen($res < 1)) $res = $wgDBname;  
		return $res;
	}
	
	/**
	 * Provides anonymous user id for current user
	 * 
	 * This is 
	 * - for anonymous users: their hashed IP address
	 * - for logged in users their RandomUserID generated by Woogle or - if this is null - their hashed MediaWiki user name
	 * 
	 * @return 		String, anonymous userID
	 */
	public static function getAnonUserID() {
		global $wgUser;
		WoogleUtil::profileIn(__METHOD__);
		$res = '';
		
		if (isset($wgUser)){
			if ($wgUser->isAnon()) {
				$res = 'i_' . md5(self::getSecretKey() . wfGetIP());	// returns IP in case of anon user
			} else {
				$uid =  WoogleConfig::getPref('Privacy', 'RandomUserID');
				if (!empty($uid)){
					$res = $uid;
				} else {
					$res = 'u_' . md5(self::getSecretKey() . $wgUser->getName());
					WoogleUtil::error('getAnonUserID: RandomUserID not set');
				}
			} 
		} else {
			WoogleUtil::error('getAnonUserID: $wgUser not set');
		}
		
		return WoogleUtil::profileOut(__METHOD__, $res);
	}
	
}

?>